{% load i18n %}
<div class="btn-toolbar">
    <div class="dropdown">
        <button class="btn btn-secondary dropdown-toggle" type="button" id="dropdownMenuButton" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
            {% trans "Actions" %}
        </button>
        <div class="dropdown-menu dropdown-menu-right" aria-labelledby="dropdownMenuButton">
            <a
                class="dropdown-item"
                {% if record.is_complete %}
                    href="{% url 'dashboard:reports-download' record.uuid %}"
                {% else %}
                    disabled
                {% endif %}
            >
                {% trans "Download" %}
            </a>
            <a
                class="dropdown-item"
                href="{% url 'dashboard:reports-delete' record.uuid %}"
            >
                {% trans "Delete" %}
            </a>
        </div>
    </div>
</div>
